###############################################################################
# Copyright (c) 2015-2019, Lawrence Livermore National Security, LLC.
#
# Produced at the Lawrence Livermore National Laboratory
#
# LLNL-CODE-716457
#
# All rights reserved.
#
# This file is part of Ascent.
#
# For details, see: http://ascent.readthedocs.io/.
#
# Please also read ascent/LICENSE
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the disclaimer below.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the disclaimer (as noted below) in the
#   documentation and/or other materials provided with the distribution.
#
# * Neither the name of the LLNS/LLNL nor the names of its contributors may
#   be used to endorse or promote products derived from this software without
#   specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL LAWRENCE LIVERMORE NATIONAL SECURITY,
# LLC, THE U.S. DEPARTMENT OF ENERGY OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
###############################################################################

###############################################################################
# Note:
# This file is named AscentConfig.cmake because once upon a time
# when it was named ascent-config.cmake, we found that CMake's exported
# targets script includes all "ascent-*.cmake" files. This logic would
# cause this script to be included more than once, seeding instability
# that caused great harm to the kingdom.
###############################################################################

cmake_minimum_required(VERSION 3.0 FATAL_ERROR)


####### Expanded from @PACKAGE_INIT@ by configure_package_config_file() #######
####### Any changes to this file will be overwritten by the next CMake run ####
####### The input file was AscentConfig.cmake.in                            ########

get_filename_component(PACKAGE_PREFIX_DIR "${CMAKE_CURRENT_LIST_DIR}/../../../" ABSOLUTE)

macro(set_and_check _var _file)
  set(${_var} "${_file}")
  if(NOT EXISTS "${_file}")
    message(FATAL_ERROR "File or directory ${_file} referenced by variable ${_var} does not exist !")
  endif()
endmacro()

macro(check_required_components _NAME)
  foreach(comp ${${_NAME}_FIND_COMPONENTS})
    if(NOT ${_NAME}_${comp}_FOUND)
      if(${_NAME}_FIND_REQUIRED_${comp})
        set(${_NAME}_FOUND FALSE)
      endif()
    endif()
  endforeach()
endmacro()

####################################################################################


if(NOT ASCENT_FOUND)

    set(ASCENT_VERSION "0.7.0")
    set(ASCENT_INSTALL_PREFIX "/usr/local")
    set(ASCENT_CONDUIT_DIR  "/usr/local")

    # advertise if fortran support is enabled
    set(ASCENT_FORTRAN_ENABLED ON)

    # advertise if vtk-h support is enabled
    set(ASCENT_VTKH_ENABLED )
    set(ASCENT_VTKH_DIR "")
    set(ASCENT_VTKM_DIR "")

    # advertise if mfem support is enabled
    set(ASCENT_MFEM_ENABLED )
    set(ASCENT_MFEM_DIR "")

    # advertise if devil ray support is enabled
    set(ASCENT_DRAY_ENABLED )
    set(ASCENT_DRAY_DIR "")

    # advertise if python support is enabled
    set(ASCENT_PYTHON_ENABLED )
    set(ASCENT_PYTHON_EXECUTABLE "")

    set(ASCENT_SERIAL_ENABLED ON)
    set(ASCENT_MPI_ENABLED OFF)

    # pull in vars with details about configured paths
    get_filename_component(ASCENT_CMAKE_CONFIG_DIR "${CMAKE_CURRENT_LIST_FILE}" PATH)

    # setup dependent pkgs (conduit + vtk-h)
    include(${ASCENT_CMAKE_CONFIG_DIR}/ascent_setup_deps.cmake)

    # include the main exported targets
    include("${ASCENT_CMAKE_CONFIG_DIR}/ascent.cmake")

    # finish setup
    include("${ASCENT_CMAKE_CONFIG_DIR}/ascent_setup_targets.cmake")

    set(ASCENT_FOUND TRUE)

endif()
